using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using Autofac;
using FitConnect;
using FitConnect.Encryption;
using FitConnect.Models;
using FitConnect.Services.Interfaces;
using FitConnect.Services.Models.v1.Submission;
using Microsoft.Extensions.Logging;
using MockContainer;
using Moq;
using Newtonsoft.Json;

namespace BasicUnitTest.Mocking;

public static class MockDependencyContainer {
    public static IContainer Container => _container ??= Create();

    private static IContainer? _container;

    public static ContainerBuilder Builder() {
        var builder = new ContainerBuilder();
        var settings = CreateEncryptionSettings(builder);

        builder.RegisterInstance(new Mock<IOAuthService>().Object);
        builder.RegisterInstance(new Mock<IRouteService>().Object);
        builder.RegisterInstance(new Mock<ICasesService>().Object);
        builder.RegisterInstance<IFitConnectSettings>(settings);
        builder.RegisterInstance(new Mock<IDestinationService>().Object);
        builder.RegisterInstance(new Mock<ISubmissionService>().Object);
        builder.RegisterInstance(new Mock<ILogger>().Object);
        return builder;
    }

    private static IContainer Create() {
        return Builder().Build();
    }


    private static MockSettings? GetSettingsFromEnvironment() {
        Console.WriteLine("Getting settings from environment");
        var privateKeyDecryption =
            Environment.GetEnvironmentVariable("MOCK_PRIVATE_KEY_DECRYPTION")!;
        var privateKeySigning = Environment.GetEnvironmentVariable("MOCK_PRIVATE_KEY_SIGNING")!;
        var publicKeyEncryption = Environment.GetEnvironmentVariable("MOCK_PUBLIC_KEY_ENCRYPTION")!;
        var publicKeySignature = Environment.GetEnvironmentVariable("MOCK_PUBLIC_KEY_SIGNATURE")!;
        var setPublicKeys = Environment.GetEnvironmentVariable("MOCK_SET_PUBLIC_KEYS")!;
        var senderClientId = Environment.GetEnvironmentVariable("MOCK_SENDER_CLIENT_ID")!;
        var senderClientSecret = Environment.GetEnvironmentVariable("MOCK_SENDER_CLIENT_SECRET")!;
        var subscriberClientId = Environment.GetEnvironmentVariable("MOCK_SUBSCRIBER_CLIENT_ID")!;
        var subscriberClientSecret =
            Environment.GetEnvironmentVariable("MOCK_SUBSCRIBER_CLIENT_SECRET")!;
        var destinationId = Environment.GetEnvironmentVariable("MOCK_DESTINATION_ID")!;
        var leikaKey = Environment.GetEnvironmentVariable("MOCK_LEIKA_KEY")!;
        var callbackSecret = Environment.GetEnvironmentVariable("MOCK_CALLBACK_SECRET")!;

        return new MockSettings(
            privateKeyDecryption, privateKeySigning,
            publicKeyEncryption, publicKeySignature,
            senderClientId, senderClientSecret,
            subscriberClientId, subscriberClientSecret,
            destinationId, leikaKey, callbackSecret, setPublicKeys);
    }

    private static MockSettings? GetSettingsFromFiles() {
        Console.WriteLine("Reading settings from files");
        try {
            var privateKeyDecryption =
                File.ReadAllText("./encryptionKeys/privateKey_decryption.json");
            var privateKeySigning = File.ReadAllText("./encryptionKeys/privateKey_signing.json");
            var publicKeyEncryption =
                File.ReadAllText("./encryptionKeys/publicKey_encryption.json");
            var publicKeySignature =
                File.ReadAllText("./encryptionKeys/publicKey_signature_verification.json");
            var setPublicKeys = File.ReadAllText("./encryptionKeys/set-public-keys.json");

            var credentials =
                JsonConvert.DeserializeObject<dynamic>(
                    File.ReadAllText("./encryptionKeys/credentials.json"))!;

            var senderClientId = (string)credentials.sender.clientId;
            var senderClientSecret = (string)credentials.sender.clientSecret;
            var subscriberClientId = (string)credentials.subscriber.clientId;
            var subscriberClientSecret = (string)credentials.subscriber.clientSecret;
            var destinationId = (string)credentials.destinationId;
            var leikaKey = (string)credentials.leikaKey;
            var callbackSecret = (string)credentials.callbackSecret;

            return new MockSettings(
                privateKeyDecryption, privateKeySigning,
                publicKeyEncryption, publicKeySignature,
                senderClientId, senderClientSecret,
                subscriberClientId, subscriberClientSecret,
                destinationId, leikaKey, callbackSecret, setPublicKeys);
        }
        catch {
            Console.WriteLine("Files not found, using environment variables");
            return null;
        }
    }


    private static MockSettings CreateEncryptionSettings(ContainerBuilder builder) {
        var settings = Environment.GetEnvironmentVariable("ENVIRONMENT_DRIVEN_TEST") == null
            ? GetSettingsFromFiles()
            : GetSettingsFromEnvironment();
        if (settings == null)
            throw new ArgumentException("Could not find settings file nor environment variables");

        builder.Register(c => settings)
            .As<MockSettings>();
        builder.Register(c => settings.ToKeySet()).As<KeySet>();

        return settings;
    }
}
